
# q-expansion of newform 1008.3.d.d, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1008
weight = 3
poly_data = [9, 0, 8, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 2, 0, 0], 1], [[0, 5, 0, 1], 1], [[4, 0, 1, 0], 1]]

hecke_ring_character_values = [[127, [1, 0, 0, 0]], [757, [1, 0, 0, 0]], [785, [-1, 0, 0, 0]], [577, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 1, 0, 0], [0, 0, 0, -1], [0, 2, -1, 0], [6, 0, 0, -4], [0, -3, -6, 0], [-6, 0, 0, 10], [0, 6, 5, 0], [0, -2, 7, 0], [38, 0, 0, 2], [-32, 0, 0, 4], [0, 11, 8, 0], [-20, 0, 0, -12], [0, 6, -2, 0], [0, 12, 3, 0], [0, 6, 14, 0], [-12, 0, 0, 38], [6, 0, 0, 28], [0, 6, -9, 0], [4, 0, 0, -14], [22, 0, 0, 0], [0, 6, 4, 0], [0, -5, 20, 0], [40, 0, 0, -2], [0, 47, 18, 0], [-110, 0, 0, -6], [0, 14, -5, 0], [-44, 0, 0, 16], [0, 16, -9, 0], [38, 0, 0, 44], [0, -4, -36, 0], [0, -42, -25, 0], [176, 0, 0, 8], [0, -24, -11, 0], [100, 0, 0, 12], [12, 0, 0, 26], [54, 0, 0, 4], [0, 26, 30, 0], [0, 17, 30, 0], [0, 22, 47, 0], [234, 0, 0, -40], [0, -46, -7, 0], [-160, 0, 0, 24], [0, 28, -7, 0], [32, 0, 0, -4], [-216, 0, 0, -56], [24, 0, 0, -68], [0, -54, 34, 0], [134, 0, 0, 36], [0, -6, -51, 0], [0, -6, -61, 0], [-4, 0, 0, -18], [0, -40, 10, 0], [0, -43, -52, 0], [0, -34, -7, 0], [0, -33, -60, 0], [90, 0, 0, -86], [186, 0, 0, 56], [0, -70, -3, 0], [402, 0, 0, 10], [0, 11, 86, 0], [-18, 0, 0, -158], [0, -18, 50, 0], [-70, 0, 0, -60], [0, 84, -25, 0], [40, 0, 0, -144], [240, 0, 0, -76], [0, -14, -53, 0], [264, 0, 0, 86], [0, -71, -26, 0], [0, -14, -43, 0], [-172, 0, 0, -100], [-10, 0, 0, 144], [-112, 0, 0, 104], [0, -48, -28, 0], [0, -18, -7, 0], [-428, 0, 0, -122], [0, -138, -53, 0], [-64, 0, 0, -54], [0, 60, 86, 0], [-442, 0, 0, -48], [0, 78, 105, 0], [486, 0, 0, -4], [-208, 0, 0, 8], [0, 90, -89, 0], [0, -84, -129, 0], [384, 0, 0, -160], [0, 101, 54, 0], [-414, 0, 0, 124], [0, -60, 114, 0], [0, 16, -90, 0], [36, 0, 0, -140], [0, 178, -11, 0], [-180, 0, 0, -20], [0, -78, 120, 0], [0, -37, 20, 0], [0, 57, 126, 0], [120, 0, 0, -152], [-142, 0, 0, 144], [654, 0, 0, -104], [0, 64, 185, 0], [0, -166, 6, 0], [0, 10, -47, 0], [34, 0, 0, 264], [270, 0, 0, 68], [0, 26, 182, 0], [0, 141, -30, 0], [0, 210, 89, 0], [394, 0, 0, -284], [460, 0, 0, -36], [366, 0, 0, 116], [0, 30, -143, 0], [-72, 0, 0, 244], [-122, 0, 0, -204], [0, 234, 125, 0], [58, 0, 0, 114], [0, 70, -70, 0], [0, 174, -17, 0], [0, -54, -239, 0], [-432, 0, 0, 146], [-420, 0, 0, -40], [0, 45, -250, 0], [0, 78, 9, 0], [-80, 0, 0, 84], [0, 162, 27, 0], [-188, 0, 0, -152], [0, -156, -156, 0], [586, 0, 0, -6], [58, 0, 0, -80], [2, 0, 0, -172], [0, 138, 17, 0], [-312, 0, 0, 184], [548, 0, 0, -312], [0, 9, -68, 0], [950, 0, 0, 48], [0, 17, -86, 0], [804, 0, 0, 184], [0, -119, 78, 0], [0, -300, -27, 0], [328, 0, 0, 12], [0, -124, -63, 0], [110, 0, 0, 344], [0, 54, -147, 0], [-430, 0, 0, -288], [0, 320, 98, 0], [-112, 0, 0, 18], [0, -95, -8, 0], [-146, 0, 0, 138], [0, 42, -27, 0], [-438, 0, 0, -244], [0, 105, 42, 0], [-620, 0, 0, -300], [0, 226, 142, 0], [4, 0, 0, 36], [0, -270, 7, 0], [-740, 0, 0, 252], [0, -79, 252, 0], [-446, 0, 0, 112], [0, -7, 20, 0], [0, -326, -237, 0], [0, 140, 85, 0], [1098, 0, 0, 196], [0, -54, -120, 0], [0, -54, 25, 0], [0, -198, -236, 0], [196, 0, 0, -180], [996, 0, 0, -70], [834, 0, 0, -208], [0, -229, 134, 0], [0, -42, 349, 0], [808, 0, 0, 214], [0, -174, -289, 0], [-286, 0, 0, -132], [-200, 0, 0, -60], [0, 179, -10, 0], [-1242, 0, 0, 64], [0, -86, -283, 0], [446, 0, 0, -94], [900, 0, 0, -226], [-494, 0, 0, -456], [0, -60, -80, 0], [-262, 0, 0, -432], [0, 59, -86, 0], [0, -102, 245, 0], [0, 131, 30, 0], [-1344, 0, 0, -136], [-156, 0, 0, -716], [-450, 0, 0, 236], [0, 14, -308, 0], [408, 0, 0, 146], [0, -138, 31, 0], [472, 0, 0, -96], [0, 237, 254, 0], [0, 38, -365, 0], [0, -315, 34, 0], [-220, 0, 0, 624], [58, 0, 0, 160], [0, -195, -76, 0], [0, -318, -222, 0], [0, -244, -235, 0], [964, 0, 0, -96], [-1394, 0, 0, -356], [960, 0, 0, -394], [0, -224, -160, 0], [0, -355, -98, 0], [-1324, 0, 0, -16], [0, -6, 527, 0], [0, -360, 65, 0], [1638, 0, 0, -182], [-218, 0, 0, -224], [0, -333, -152, 0], [420, 0, 0, 508], [0, 308, 236, 0], [0, 256, -62, 0], [1242, 0, 0, -172], [-620, 0, 0, 36], [0, 313, -242, 0], [0, 218, 211, 0], [0, -126, 175, 0], [-1468, 0, 0, -216], [-1518, 0, 0, -230], [0, -148, 29, 0], [-992, 0, 0, -504], [0, -686, -212, 0], [-544, 0, 0, 468], [0, 43, -8, 0], [0, -674, -223, 0], [598, 0, 0, -666], [0, 210, 289, 0], [1476, 0, 0, -448], [1278, 0, 0, 562], [-572, 0, 0, 60], [0, 412, 423, 0], [-1618, 0, 0, 338], [0, -516, -308, 0], [492, 0, 0, 842], [0, -328, -299, 0], [0, -6, -253, 0], [0, 598, 236, 0], [0, -42, -419, 0], [2132, 0, 0, -88], [-496, 0, 0, 716], [-1296, 0, 0, 176], [0, 45, 268, 0], [0, -16, -94, 0], [-18, 0, 0, 778], [0, 286, 370, 0], [-1158, 0, 0, -464], [0, -42, 417, 0], [272, 0, 0, 24], [0, 237, 186, 0], [0, -250, 375, 0], [-1308, 0, 0, 326], [0, -401, 10, 0], [0, -590, -19, 0], [-802, 0, 0, -312], [470, 0, 0, -94], [0, 267, -174, 0], [1166, 0, 0, -408], [802, 0, 0, -312], [0, -98, -339, 0], [662, 0, 0, -240], [380, 0, 0, -558], [0, 401, 206, 0], [-1292, 0, 0, -576], [0, -24, 435, 0], [0, 103, -446, 0], [1722, 0, 0, 256], [0, 322, 285, 0], [240, 0, 0, -454], [-1448, 0, 0, -336], [828, 0, 0, 566], [2224, 0, 0, -432], [-1692, 0, 0, 74], [814, 0, 0, 822], [0, 182, 285, 0], [-248, 0, 0, 376], [2352, 0, 0, 200], [0, -74, 258, 0], [0, 34, -644, 0], [-1078, 0, 0, 164], [0, -422, 36, 0], [-1056, 0, 0, 902], [-1052, 0, 0, 252], [0, 102, -129, 0], [-48, 0, 0, 296], [0, -372, -449, 0], [-3266, 0, 0, 30], [0, -237, -276, 0], [0, 28, -587, 0], [0, -412, 136, 0], [0, -390, 361, 0], [0, 822, 422, 0], [-1326, 0, 0, 776], [0, 703, 102, 0], [-636, 0, 0, 664], [0, -169, -468, 0], [0, 634, 434, 0], [-1700, 0, 0, 516], [-2870, 0, 0, 148], [0, 366, 627, 0], [654, 0, 0, 340]]
